@@ -80,7 +80,7 @@ digest_context::digest_context(unsigned char *key, int key_len,
 
 volatile bool	g_sasl_scram_sha_got_first_msg = false;
 
-static char	*complete_nonce = NULL;
+static STRING	 complete_nonce = NULL;
 static char	 nonce[64] = { '\0' };
 
 static unsigned char	signature_expected[EVP_MAX_MD_SIZE] = { '\0' };
@@ -90,7 +90,7 @@ static unsigned int	signature_expected_len = 0;
 /*lint -sem(get_salted_password, r_null) */
 
 static void
-generate_and_store_nonce()
+generate_and_store_nonce(void)
 {
 	static const char legal_index[] =
 	    "!\"#$%&'()*+-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`"
@@ -111,11 +111,11 @@ generate_and_store_nonce()
 #endif
 
 	nonce[ARRAY_SIZE(nonce) - 1] = '\0';
-	debug("generate_and_store_nonce: nonce: %s", nonce);
+	debug("%s: nonce: %s", __func__, nonce);
 }
 
-static const char *
-get_encoded_msg(const char *source)
+static CSTRING
+get_encoded_msg(CSTRING source)
 {
 	static char	encoded_msg[4096] = { '\0' };
 
@@ -129,9 +129,9 @@ get_encoded_msg(const char *source)
 int
 sasl_scram_sha_send_client_first_msg(void)
 {
-	char		*msg;
-	const char	*encoded_msg;
-	const char	*username = Config("sasl_username");
+	CSTRING encoded_msg;
+	CSTRING username = Config("sasl_username");
+	STRING msg;
 
 	if (!is_valid_username(username))
 		return -1;
@@ -151,9 +151,9 @@ sasl_scram_sha_send_client_first_msg(void)
 /* C: c=biws,r=rOprNGfwEbeRWgbNEkqO%hvYDpWUa2RaTCAfuxFIlj)hNlF$k0,
       p=dHzbZapWIk4jUhN+Ute9ytag9zjfMHgsqmmiz7AndVQ= */
 int
-sasl_scram_sha_send_client_final_msg(const char *proof)
+sasl_scram_sha_send_client_final_msg(CSTRING proof)
 {
-	char		*cli_final_msg;
+	STRING		 cli_final_msg;
 	size_t		 size;
 	std::string	 str("c=biws,r=");
 
@@ -172,15 +172,15 @@ sasl_scram_sha_send_client_final_msg(const char *proof)
 		return -1;
 	}
 
-	debug("sasl_scram_sha_send_client_final_msg: C: %s", cli_final_msg);
+	debug("%s: C: %s", __func__, cli_final_msg);
 	delete[] cli_final_msg;
 	return 0;
 }
 
-static char *
-get_decoded_msg(const char *source, int *outlen) noexcept
+static STRING
+get_decoded_msg(CSTRING source, int *outlen) noexcept
 {
-	char	*decoded_msg = NULL;
+	STRING	 decoded_msg = NULL;
 	int	 length_needed = b64_decode(source, NULL, 0);
 
 	if (length_needed < 0)
@@ -204,24 +204,27 @@ get_decoded_msg(const char *source, int *outlen) noexcept
 /* S: r=rOprNGfwEbeRWgbNEkqO%hvYDpWUa2RaTCAfuxFIlj)hNlF$k0,
       s=W22ZaJ0SNY7soEsUEjb6gQ==,i=4096 */
 static int
-get_sfm_components(const char *msg, unsigned char **salt, int *saltlen,
+get_sfm_components(CSTRING msg, unsigned char **salt, int *saltlen,
     int *iter)
 {
+	STRING	 decoded_msg = NULL;
 	bool	 ok = false;
-	char	*decoded_msg = NULL;
 
 	*salt = NULL;
 	*saltlen = 0;
 	*iter = PKCS5_DEFAULT_ITER;
 
 	try {
+		char *cp, *b64salt;
+		size_t n;
+
 		if ((decoded_msg = get_decoded_msg(msg, NULL)) == NULL) {
 			throw std::runtime_error("unable to get decoded "
 			    "message");
 		}
 
-		debug("get_sfm_components: S: %s", decoded_msg);
-		char *cp = decoded_msg;
+		debug("%s: S: %s", __func__, decoded_msg);
+		cp = decoded_msg;
 
 		if (strncmp(cp, "r=", 2) != STRINGS_MATCH)
 			throw std::runtime_error("expected nonce");
@@ -231,7 +234,7 @@ get_sfm_components(const char *msg, unsigned char **salt, int *saltlen,
 		if (strncmp(cp, nonce, strlen(nonce)) != STRINGS_MATCH)
 			throw std::runtime_error("nonce mismatch");
 
-		size_t n = strcspn(cp, ",");
+		n = strcspn(cp, ",");
 		cp[n] = '\0';
 		free(complete_nonce);
 		complete_nonce = sw_strdup(cp);
@@ -243,11 +246,10 @@ get_sfm_components(const char *msg, unsigned char **salt, int *saltlen,
 		cp += 3;
 		n = strcspn(cp, ",");
 		cp[n] = '\0';
-		char *b64salt = sw_strdup(cp);
+		b64salt = sw_strdup(cp);
 		cp[n] = ',';
-		*salt =
-		    reinterpret_cast<unsigned char *>(get_decoded_msg(b64salt,
-		    saltlen));
+		*salt = reinterpret_cast<unsigned char *>
+		    (get_decoded_msg(b64salt, saltlen));
 		free(b64salt);
 
 		if (*salt == NULL)
@@ -267,7 +269,7 @@ get_sfm_components(const char *msg, unsigned char **salt, int *saltlen,
 		*salt = NULL;
 		*saltlen = 0;
 		*iter = PKCS5_DEFAULT_ITER;
-		err_log(0, "get_sfm_components: %s", e.what());
+		err_log(0, "%s: %s", __func__, e.what());
 	}
 
 	if (decoded_msg)
@@ -285,7 +287,7 @@ get_salted_password(const unsigned char *salt, int saltlen, int iter,
 	unsigned char *out = NULL;
 
 	try {
-		const char *pass;
+		CSTRING pass;
 
 		if (*outsize = EVP_MD_size(EVP_sha256()), *outsize < 0) {
 			throw std::runtime_error("message digest size "
@@ -305,10 +307,10 @@ get_salted_password(const unsigned char *salt, int saltlen, int iter,
 	} catch (const std::runtime_error &e) {
 		*outsize = 0;
 		delete[] out;
-		err_log(0, "get_salted_password: %s", e.what());
+		err_log(0, "%s: %s", __func__, e.what());
 		return NULL;
 	} catch (const std::bad_alloc &e) {
-		err_exit(ENOMEM, "get_salted_password: %s", e.what());
+		err_exit(ENOMEM, "%s: %s", __func__, e.what());
 		/* NOTREACHED */
 	} catch (...) {
 		sw_assert_not_reached();
@@ -326,10 +328,10 @@ get_digest(struct digest_context *ctx)
 	return 0;
 }
 
-static char *
+static STRING
 get_client_first_msg_bare() noexcept
 {
-	char		*msg_bare;
+	STRING		 msg_bare;
 	size_t		 size;
 	std::string	 str("n=");
 
@@ -344,10 +346,10 @@ get_client_first_msg_bare() noexcept
 	return msg_bare;
 }
 
-static char *
+static STRING
 get_client_final_msg_wo_proof() noexcept
 {
-	char		*msg_wo_proof;
+	STRING		 msg_wo_proof;
 	size_t		 size;
 	std::string	 str("c=biws,r=");
 
@@ -367,10 +369,10 @@ get_client_final_msg_wo_proof() noexcept
  *              client-final-message-without-proof
  */
 static unsigned char *
-get_auth_msg(const char *b64msg, size_t *auth_msg_len)
+get_auth_msg(CSTRING b64msg, size_t *auth_msg_len)
 {
-	char	*msg_bare, *msg_wo_proof, *serv_first_msg;
-	char	*out;
+	STRING msg_bare, msg_wo_proof, serv_first_msg;
+	STRING out;
 
 	msg_bare = get_client_first_msg_bare();
 	msg_wo_proof = get_client_final_msg_wo_proof();
@@ -387,7 +389,7 @@ get_auth_msg(const char *b64msg, size_t *auth_msg_len)
 }
 
 int
-sasl_scram_sha_handle_serv_first_msg(const char *msg)
+sasl_scram_sha_handle_serv_first_msg(CSTRING msg)
 {
 	char		 proof[EVP_MAX_MD_SIZE] = { '\0' };
 	int		 iter = PKCS5_DEFAULT_ITER;
@@ -474,10 +476,10 @@ sasl_scram_sha_handle_serv_first_msg(const char *msg)
  * S: v=6rriTRBi23WpRR/wtup+mMhUZUn/dB5nLTJRsjl95G4=
  */
 int
-sasl_scram_sha_handle_serv_final_msg(const char *msg)
+sasl_scram_sha_handle_serv_final_msg(CSTRING msg)
 {
+	STRING	 decoded_msg = NULL;
 	bool	 signature_ok = false;
-	char	*decoded_msg = NULL;
 
 	try {
 		char		*cp;
@@ -490,8 +492,7 @@ sasl_scram_sha_handle_serv_final_msg(const char *msg)
 			throw std::runtime_error("expected server signature");
 		}
 
-		debug("sasl_scram_sha_handle_serv_final_msg: S: %s",
-		    decoded_msg);
+		debug("%s: S: %s", __func__, decoded_msg);
 
 		cp = decoded_msg;
 		cp += 2;
@@ -509,8 +510,7 @@ sasl_scram_sha_handle_serv_final_msg(const char *msg)
 		delete[] signature;
 	} catch (const std::runtime_error &e) {
 		delete[] decoded_msg;
-		err_log(0, "sasl_scram_sha_handle_serv_final_msg: %s",
-		    e.what());
+		err_log(0, "%s: %s", __func__, e.what());
 		return -1;
 	}
 
