@@ -2,7 +2,7 @@
 #define CMD_SQUERY_H
 
 __SWIRC_BEGIN_DECLS
-void	cmd_squery(const char *) PTR_ARGS_NONNULL;
+void	cmd_squery(const char *) NONNULL;
 __SWIRC_END_DECLS
 
 #endif
