@@ -63,8 +63,8 @@
 #include "swircpaths.h"
 #endif
 
-static bool	shouldHighlightMessage_case1(const char *) PTR_ARGS_NONNULL;
-static bool	shouldHighlightMessage_case2(const char *) PTR_ARGS_NONNULL;
+static bool	shouldHighlightMessage_case1(const char *) NONNULL;
+static bool	shouldHighlightMessage_case2(const char *) NONNULL;
 
 static void
 acknowledge_ctcp_request(const char *cmd, const struct special_msg_context *ctx)
@@ -217,7 +217,7 @@ static wchar_t *
 get_message(const wchar_t *s1, const wchar_t *s2, const wchar_t *s3,
     const wchar_t *s4, const wchar_t *s5)
 {
-	static wchar_t	message[1001] = { '\0' };
+	static wchar_t	message[1001] = { L'\0' };
 
 	(void) wmemset(message, 0L, ARRAY_SIZE(message));
 
