@@ -274,6 +274,16 @@ static struct irc_message_compo *
 		 SortMsgCompo(const char *);
 static void	 FreeMsgCompo(struct irc_message_compo *);
 
+static int
+cmp_fn(const void *vp1, const void *vp2)
+{
+	const struct numeric_events_tag *evt1, *evt2;
+
+	evt1 = vp1;
+	evt2 = vp2;
+	return strcmp(evt1->numeric_event, evt2->numeric_event);
+}
+
 /**
  * Initialize irc module
  */
@@ -281,6 +291,7 @@ void
 irc_init(void)
 {
 	char *nickname;
+	static bool is_sorted = false;
 
 	if (g_cmdline_opts->nickname)
 		irc_set_my_nickname(g_cmdline_opts->nickname);
@@ -292,6 +303,12 @@ irc_init(void)
 	else
 		err_quit("%s: no nickname", __func__);
 	event_names_init();
+
+	if (!is_sorted) {
+		qsort(numeric_events, ARRAY_SIZE(numeric_events),
+		    sizeof numeric_events[0], cmp_fn);
+		is_sorted = true;
+	}
 }
 
 /**
@@ -547,29 +564,33 @@ handle_normal_event(struct irc_message_compo *compo)
 static int
 handle_numeric_event(struct irc_message_compo *compo)
 {
-	for (struct numeric_events_tag *sp = &numeric_events[0];
-	     sp < &numeric_events[ARRAY_SIZE(numeric_events)]; sp++) {
-		if (strings_match(sp->numeric_event, compo->command)) {
-			if (sp->event_handler != NULL) {
-				sp->event_handler(compo);
-			} else {
-				if (sp->window == STATUS_WINDOW) {
-					irc_extract_msg(compo, g_status_window,
-					    sp->ext_bits,
-					    !strncmp(sp->official_name, "ERR_",
-					    4));
-				} else if (sp->window == ACTIVE_WINDOW) {
-					irc_extract_msg(compo, g_active_window,
-					    sp->ext_bits,
-					    !strncmp(sp->official_name, "ERR_",
-					    4));
-				} else {
-					sw_assert_not_reached();
-				}
-			}
+	struct numeric_events_tag *evt, key;
 
-			return 0;
+	key.numeric_event	= compo->command;
+	key.official_name	= NULL;
+	key.window		= NO_WINDOW;
+	key.ext_bits		= 0;
+	key.event_handler	= NULL;
+
+	if ((evt = bsearch(&key, numeric_events, ARRAY_SIZE(numeric_events),
+	    sizeof numeric_events[0], cmp_fn)) != NULL) {
+		if (evt->event_handler != NULL) {
+			evt->event_handler(compo);
+		} else {
+			if (evt->window == STATUS_WINDOW) {
+				irc_extract_msg(compo, g_status_window,
+				    evt->ext_bits,
+				    !strncmp(evt->official_name, "ERR_", 4));
+			} else if (evt->window == ACTIVE_WINDOW) {
+				irc_extract_msg(compo, g_active_window,
+				    evt->ext_bits,
+				    !strncmp(evt->official_name, "ERR_", 4));
+			} else {
+				sw_assert_not_reached();
+			}
 		}
+
+		return 0;
 	}
 
 	return -1;
