@@ -54,6 +54,7 @@
 #include "assertAPI.h"
 #include "curses-funcs.h"
 #include "cursesInit.h"
+#include "dataClassify.h"
 #include "errHand.h"
 #include "i18n.h"
 #include "io-loop.h"
@@ -84,7 +85,7 @@
 /* Things with external linkage
    ============================ */
 
-chararray_t	g_swircVersion	= "v3.4.3";
+chararray_t	g_swircVersion	= "v3.4.4";
 chararray_t	g_swircYear	= "2012-2023";
 chararray_t	g_swircAuthor	= "Markus Uhlin";
 chararray_t	g_swircWebAddr	= "https://www.nifty-networks.net/swirc/";
@@ -737,6 +738,12 @@ main(int argc, char *argv[])
 	nestHome_deinit();
 	term_deinit();
 
+	if (isValid(g_dev_null)) {
+		if (fclose(g_dev_null) != 0)
+			err_ret("fclose");
+		else
+			g_dev_null = NULL;
+	}
 	cmdline_options_destroy();
 	puts("- Exit Success! -");
 	return (EXIT_SUCCESS);
@@ -801,12 +808,31 @@ cmdline_options_destroy(void)
 void
 redir_stderr(void)
 {
-	g_stderr_fd = dup(fileno(stderr));
-	(void) dup2(fileno(g_dev_null), fileno(stderr));
+	const bool is_valid_fd = (g_stderr_fd != -1);
+
+	if (is_valid_fd)
+		err_log(EBUSY, "%s: valid file descriptor", __func__);
+	else if (!isValid(g_dev_null))
+		err_log(EFAULT, "%s: dev null not open", __func__);
+	else if ((g_stderr_fd = dup(fileno(stderr))) == -1)
+		err_sys("%s: dup() error", __func__);
+	else if (dup2(fileno(g_dev_null), fileno(stderr)) == -1)
+		err_sys("%s: dup2() error", __func__);
+	else
+		return;
 }
 
 void
 restore_stderr(void)
 {
-	(void) dup2(g_stderr_fd, fileno(stderr));
+	const bool is_valid_fd = (g_stderr_fd != -1);
+
+	if (!is_valid_fd)
+		err_log(EBADF, "%s", __func__);
+	else if (dup2(g_stderr_fd, fileno(stderr)) == -1)
+		err_sys("%s: dup2() error", __func__);
+	else if (close(g_stderr_fd) != 0)
+		err_log(errno, "%s: close() error", __func__);
+	else
+		g_stderr_fd = -1;
 }
