@@ -259,8 +259,7 @@ static struct ptext_colorMap_tag {
 *                                                               *
 ****************************************************************/
 
-static size_t	 get_mb_strlen(CSTRING) PTR_ARGS_NONNULL;
-static char	*get_buffer(CSTRING) PTR_ARGS_NONNULL;
+static char *get_buffer(CSTRING) NONNULL;
 
 static void
 addmbs(WINDOW *win, const unsigned char *mbs)
@@ -699,7 +698,7 @@ new_row(WINDOW *win, int *insert_count, int *rep_count)
 	WADDCH(win, '\n');
 	*insert_count = 0;
 
-	if (rep_count != NULL)
+	if (isValid(rep_count))
 		(*rep_count) ++;
 }
 
@@ -759,7 +758,7 @@ case_default(const struct case_default_context *ctx, int *rep_count,
 	} else if (!start_on_a_new_row((*insert_count) + ctx->diff + 1,
 		    ctx->win)) {
 		addmbs(ctx->win, mbs);
-		(*insert_count) ++;
+		(*insert_count) += xwcwidth(ctx->wc, 2); // XXX
 	} else {
 		/*
 		 * Start on a new row
@@ -778,7 +777,7 @@ case_default(const struct case_default_context *ctx, int *rep_count,
 
 		if (ctx->wc != L' ') {
 			addmbs(ctx->win, mbs);
-			(*insert_count) ++;
+			(*insert_count) += xwcwidth(ctx->wc, 2);
 		}
 	}
 
@@ -823,45 +822,27 @@ case_underline(WINDOW *win, bool *is_underline)
 	}
 }
 
-/**
- * Get multibyte string length
- */
-static size_t
-get_mb_strlen(CSTRING s)
-{
-	const size_t ERR_CASE1 = static_cast<size_t>(-1);
-	const size_t ERR_CASE2 = static_cast<size_t>(-2);
-	size_t idx = 0;
-	size_t len = 0;
-
-	while (true) {
-		const size_t ret = mbrlen(&s[idx], MB_CUR_MAX, NULL);
-
-		if (ret == ERR_CASE1 || ret == ERR_CASE2) {
-			return strlen(s);
-		} else if (ret == 0) {
-			break;
-		} else {
-			idx += ret;
-			len += 1;
-		}
-	}
-
-	return len;
-}
-
 static void
 set_indent(int *indent, CSTRING fmt, ...)
 {
-	va_list		 ap;
 	char		*str = NULL;
+	size_t		 bytes_convert;
+	va_list		 ap;
+	wchar_t		 wcs[400] = { L'\0' };
 
 	va_start(ap, fmt);
 	str = strdup_vprintf(fmt, ap);
 	va_end(ap);
 
-	*indent = static_cast<int>(get_mb_strlen(squeeze_text_deco(str)));
+	bytes_convert = xmbstowcs(addrof(wcs[0]), squeeze_text_deco(str),
+	    ARRAY_SIZE(wcs) - 1);
+	wcs[ARRAY_SIZE(wcs) - 1] = L'\0';
 	free(str);
+	if (bytes_convert == g_conversion_failed) {
+		*indent = 0;
+		return;
+	}
+	*indent = xwcswidth(addrof(wcs[0]), 2);
 }
 
 /**
@@ -1069,7 +1050,7 @@ try_convert_buf_with_cs(CSTRING buf, CSTRING codeset)
 			throw std::runtime_error("conversion failed");
 		}
 
-		if (bytes_convert == (size - 1))
+		if (bytes_convert >= (size - 1))
 			out[size - 1] = 0L;
 		if (xsetlocale(LC_CTYPE, original_locale) == NULL) {
 			err_log(EPERM, "%s: cannot restore original "
@@ -1405,25 +1386,22 @@ static char *
 get_buffer(CSTRING orig)
 {
 #ifdef HAVE_LIBICONV
+#if defined(__cplusplus) && __cplusplus >= 201103L
+#define fromcode_pb(_str) fromcode.emplace_back(_str)
+#else
+#define fromcode_pb(_str) fromcode.push_back(_str)
+#endif
 #define UTF8_MAXBYTE 4
 	if (!config_bool("iconv_conversion", false))
 		return sw_strdup(orig);
 
 	std::list<std::string> fromcode;
 
-#if defined(__cplusplus) && __cplusplus >= 201103L
-	fromcode.emplace_back("UTF-8");
-	fromcode.emplace_back("ISO-8859-1");
-	fromcode.emplace_back("ISO-8859-15");
-	fromcode.emplace_back("WINDOWS-1251");
-	fromcode.emplace_back("WINDOWS-1252");
-#else
-	fromcode.push_back("UTF-8");
-	fromcode.push_back("ISO-8859-1");
-	fromcode.push_back("ISO-8859-15");
-	fromcode.push_back("WINDOWS-1251");
-	fromcode.push_back("WINDOWS-1252");
-#endif
+	fromcode_pb("UTF-8");
+	fromcode_pb("ISO-8859-1");
+	fromcode_pb("ISO-8859-15");
+	fromcode_pb("WINDOWS-1251");
+	fromcode_pb("WINDOWS-1252");
 
 	for (const std::string &x : fromcode) {
 		char *in, *orig_copy, *out, *out_p;
@@ -1502,7 +1480,7 @@ printtext_puts(WINDOW *pwin, CSTRING buf, int indent, int max_lines,
 		err_sys("%s: init_once", __func__);
 #endif
 
-	if (rep_count)
+	if (isValid(rep_count))
 		*rep_count = 0;
 	if (pwin == NULL || buf == NULL)
 		err_exit(EINVAL, "%s", __func__);
@@ -1552,8 +1530,21 @@ printtext_puts(WINDOW *pwin, CSTRING buf, int indent, int max_lines,
 			wchar_t		*wcp = NULL;
 
 			if (wc == L' ' && (wcp = wcschr(wc_bufp + 1, L' ')) !=
-			    NULL)
-				diff = (wcp - wc_bufp);
+			    NULL) {
+				static wchar_t		str[4096] = { L'\0' };
+				static const ptrdiff_t	ARSZ = static_cast
+				    <ptrdiff_t>(ARRAY_SIZE(str));
+
+				if ((diff = (wcp - wc_bufp)) > ARSZ - 1)
+					diff = ARSZ - 1;
+				sw_assert(diff > 0);
+				(void) wcsncpy(str, wc_bufp, diff);
+				str[diff] = L'\0';
+				if ((diff = xwcswidth(wcspbrk(str,
+				    L"\035\002\003\017\026\037") ?
+				    squeeze_text_deco_wide(str) : str, 2)) < 0)
+					diff = 0;
+			}
 			struct case_default_context def_ctx(pwin, wc,
 			    !wcscmp(wc_bufp + 1, L""), indent, max_lines, diff);
 			case_default(&def_ctx, rep_count, &lines_count,
