@@ -57,6 +57,16 @@
 
 #include "commands/misc.h"
 
+/*
+ * Full-width length, i.e. the number of column positions required to
+ * display a full width wide character. (Windows behaves differently.)
+ */
+#if defined(UNIX)
+#define FWLEN 2
+#elif defined(WIN32)
+#define FWLEN 1
+#endif
+
 /****************************************************************
 *                                                               *
 *  -------------- Objects with external linkage --------------  *
@@ -147,12 +157,31 @@ write_cmdprompt(WINDOW *win, CSTRING prompt, int size)
 	mutex_unlock(&g_puts_mutex);
 
 	if (ret == ERR)
-		readline_error(0, "write_cmdprompt: werase");
+		readline_ferror(0, "%s: werase", __func__);
 
 	printtext_puts(win, prompt, -1, -1, NULL);
 	UNUSED_PARAM(size);
 }
 
+static int
+get_subtrahend(const volatile struct readline_session_context *ctx,
+    const int diff)
+{
+	const wchar_t	*ptr;
+	int		 i, j;
+
+	i = j = 0;
+	ptr = addrof(ctx->buffer[ctx->bufpos - 1]);
+
+	while (i < diff && ptr >= addrof(ctx->buffer[0])) {
+		i += readline_wcwidth(*ptr, FWLEN);
+		j++;
+		ptr--;
+	}
+
+	return j;
+}
+
 /**
  * When swapping between panels: computes the new window entry.
  */
@@ -160,24 +189,26 @@ static void
 compute_new_window_entry(const volatile struct readline_session_context *ctx,
 			 bool fwd)
 {
-	int bufindex, diff;
-	wchar_t *str1, *str2;
+	const wchar_t	*str1, *str2;
+	int		 bufindex, diff;
 
 	if (fwd) {
 		diff = (COLS / 2);
-		bufindex = int_diff(ctx->bufpos, diff);
+		bufindex = int_diff(ctx->bufpos, get_subtrahend(ctx, diff));
 
-		if ((COLS % 2) == 0)
-			bufindex += 1;
+		if (bufindex < 0)
+			readline_ferror(ERANGE, "%s", __func__);
 
 		str1 = &ctx->buffer[bufindex];
 		str2 = &ctx->buffer[ctx->bufpos];
 	} else {
 		diff = int_diff(COLS / 2, ctx->prompt_size);
+		bufindex = int_diff(ctx->bufpos, get_subtrahend(ctx, diff));
 
-		if ((bufindex = int_diff(ctx->bufpos, diff)) < 0) {
-			readline_error(ERANGE, "compute_new_window_entry");
-			/* NOTREACHED */
+		if (bufindex < 0) {
+			printtext_print("warn", "%s: bufindex=%d", __func__,
+			    bufindex);
+			bufindex = 0;
 		}
 
 		str1 = &ctx->buffer[bufindex];
@@ -226,54 +257,65 @@ magic_swap_panels(volatile struct readline_session_context *ctx, bool fwd)
 static void
 case_key_backspace(volatile struct readline_session_context *ctx)
 {
-	int ret[3];
+	int ret[2] = { OK };
+	int width;
+	struct current_cursor_pos yx;
 
 	if (ctx->bufpos == 0) {
 		term_beep();
 		return;
 	}
+
 	if (loLim_isset(ctx->act, ctx->prompt_size))
 		magic_swap_panels(ctx, false);
+	yx = term_get_pos(ctx->act);
+
 	if (ctx->insert_mode) {
 		wchar_t *ptr;
 
+		width = readline_wcwidth(ctx->buffer[ctx->bufpos - 1], FWLEN);
 		ptr = &ctx->buffer[ctx->bufpos--];
 		(void) wmemmove(ptr - 1, ptr, wcslen(ptr));
 		ctx->buffer[--ctx->numins] = 0L;
 
 		mutex_lock(&g_puts_mutex);
-		ret[0] = wmove(ctx->act, term_get_pos(ctx->act).cury,
-		    term_get_pos(ctx->act).curx - 1);
-		ret[1] = wdelch(ctx->act);
-		ret[2] = wclrtoeol(ctx->act);
+		if (width > 0) {
+			ctx->vispos -= width;
+			ret[0] = wmove(ctx->act, yx.cury, yx.curx - width);
+			ret[1] = wclrtoeol(ctx->act);
+		}
 		mutex_unlock(&g_puts_mutex);
 
 		if (ret[0] == ERR)
-			readline_error(EIO, "case_key_backspace: wmove");
+			readline_ferror(EIO, "%s: wmove", __func__);
 		else if (ret[1] == ERR)
-			readline_error(EIO, "case_key_backspace: wdelch");
-		else if (ret[2] == ERR)
-			readline_error(EIO, "case_key_backspace: wclrtoeol");
+			readline_ferror(EIO, "%s: wclrtoeol", __func__);
 
-		readline_winsnstr(ctx->act, &ctx->buffer[ctx->bufpos], -1);
+		if (width > 0) {
+			readline_winsnstr(ctx->act, &ctx->buffer[ctx->bufpos],
+			    -1);
+		}
 	} else {
 		/*
 		 * Not insert mode
 		 */
 
+		width = readline_wcwidth(ctx->buffer[ctx->bufpos - 1], FWLEN);
 		ctx->buffer[--ctx->bufpos] = 0L;
 		ctx->numins--;
 
 		mutex_lock(&g_puts_mutex);
-		ret[0] = wmove(ctx->act, term_get_pos(ctx->act).cury,
-		    term_get_pos(ctx->act).curx - 1);
-		ret[1] = wdelch(ctx->act);
+		if (width > 0) {
+			ctx->vispos -= width;
+			ret[0] = wmove(ctx->act, yx.cury, yx.curx - width);
+			ret[1] = wclrtoeol(ctx->act);
+		}
 		mutex_unlock(&g_puts_mutex);
 
 		if (ret[0] == ERR)
-			readline_error(EIO, "case_key_backspace: wmove");
+			readline_ferror(EIO, "%s: wmove", __func__);
 		else if (ret[1] == ERR)
-			readline_error(EIO, "case_key_backspace: wdelch");
+			readline_ferror(EIO, "%s: wclrtoeol", __func__);
 	}
 
 	mutex_lock(&g_puts_mutex);
@@ -306,9 +348,9 @@ case_key_dc(volatile struct readline_session_context *ctx)
 	mutex_unlock(&g_puts_mutex);
 
 	if (ret[0] == ERR)
-		readline_error(EIO, "case_key_dc: wdelch");
+		readline_ferror(EIO, "%s: wdelch", __func__);
 	else if (ret[1] == ERR)
-		readline_error(EIO, "case_key_dc: wclrtoeol");
+		readline_ferror(EIO, "%s: wclrtoeol", __func__);
 
 	readline_winsnstr(ctx->act, &ctx->buffer[ctx->bufpos], -1);
 
@@ -323,6 +365,7 @@ case_key_dc(volatile struct readline_session_context *ctx)
 static void
 case_key_left(volatile struct readline_session_context *ctx)
 {
+	int width;
 	struct current_cursor_pos yx;
 
 	if (ctx->bufpos == 0) {
@@ -334,14 +377,16 @@ case_key_left(volatile struct readline_session_context *ctx)
 		magic_swap_panels(ctx, false);
 
 	mutex_lock(&g_puts_mutex);
-	ctx->bufpos--;
-	yx = term_get_pos(ctx->act);
-	if (wmove(ctx->act, yx.cury, yx.curx - 1) == ERR) {
-		mutex_unlock(&g_puts_mutex);
-		readline_error(EIO, "wmove");
-		/* NOTREACHED */
+	if ((width = readline_wcwidth(ctx->buffer[--ctx->bufpos], FWLEN)) > 0) {
+		ctx->vispos -= width;
+		yx = term_get_pos(ctx->act);
+		if (wmove(ctx->act, yx.cury, yx.curx - width) == ERR) {
+			mutex_unlock(&g_puts_mutex);
+			readline_error(EIO, "wmove");
+			/* NOTREACHED */
+		}
+		(void) wrefresh(ctx->act);
 	}
-	(void) wrefresh(ctx->act);
 	mutex_unlock(&g_puts_mutex);
 }
 
@@ -351,6 +396,7 @@ case_key_left(volatile struct readline_session_context *ctx)
 static void
 case_key_right(volatile struct readline_session_context *ctx)
 {
+	int width;
 	struct current_cursor_pos yx;
 
 	if (!ctx->insert_mode) {
@@ -362,14 +408,16 @@ case_key_right(volatile struct readline_session_context *ctx)
 		magic_swap_panels(ctx, true);
 
 	mutex_lock(&g_puts_mutex);
-	ctx->bufpos++;
-	yx = term_get_pos(ctx->act);
-	if (wmove(ctx->act, yx.cury, yx.curx + 1) == ERR) {
-		mutex_unlock(&g_puts_mutex);
-		readline_error(EIO, "wmove");
-		/* NOTREACHED */
+	if ((width = readline_wcwidth(ctx->buffer[ctx->bufpos++], FWLEN)) > 0) {
+		ctx->vispos += width;
+		yx = term_get_pos(ctx->act);
+		if (wmove(ctx->act, yx.cury, yx.curx + width) == ERR) {
+			mutex_unlock(&g_puts_mutex);
+			readline_error(EIO, "wmove");
+			/* NOTREACHED */
+		}
+		(void) wrefresh(ctx->act);
 	}
-	(void) wrefresh(ctx->act);
 	mutex_unlock(&g_puts_mutex);
 }
 
@@ -394,10 +442,10 @@ finalize_out_string(const wchar_t *buf)
 
 	if ((bytes_convert = wcstombs(out, buf, size - 1)) ==
 	    g_conversion_failed) {
-		err_log(errno, "finalize_out_string: wcstombs");
+		err_log(errno, "%s: wcstombs", __func__);
 		BZERO(out, size);
 		return xrealloc(out, 1);
-	} else if (bytes_convert == (size - 1)) {
+	} else if (bytes_convert >= (size - 1)) {
 		out[size - 1] = '\0';
 	}
 
@@ -419,7 +467,7 @@ finalize_out_string(const wchar_t *buf)
 	if (WideCharToMultiByte(CP_UTF8, 0, buf, -1, out, size, NULL, NULL) > 0)
 		return xrealloc(out, strlen(out) + 1);
 
-	err_log(errno, "finalize_out_string: WideCharToMultiByte");
+	err_log(errno, "%s: WideCharToMultiByte", __func__);
 	BZERO(out, size);
 	return xrealloc(out, 1);
 }
@@ -451,15 +499,19 @@ static void
 handle_key(volatile struct readline_session_context *ctx, wint_t wc,
     bool check_reset)
 {
+	int width;
+
 	if (ctx->no_bufspc) {
 		term_beep();
 		return;
 	}
+
 	if (hiLim_isset(ctx->act))
 		magic_swap_panels(ctx, true);
+	width = readline_wcwidth(wc, FWLEN);
+
 	if (ctx->insert_mode) {
-		int		 ret;
-		wchar_t		*ptr;
+		wchar_t *ptr;
 
 		ptr = &ctx->buffer[ctx->bufpos];
 		(void) wmemmove(ptr + 1, ptr, wcslen(ptr));
@@ -467,20 +519,28 @@ handle_key(volatile struct readline_session_context *ctx, wint_t wc,
 
 		ctx->bufpos++;
 		ctx->numins++;
+		ctx->vispos += width;
 
 		readline_winsch(ctx->act, wc);
 
-		mutex_lock(&g_puts_mutex);
-		ret = wmove(ctx->act, term_get_pos(ctx->act).cury,
-		    term_get_pos(ctx->act).curx + 1);
-		mutex_unlock(&g_puts_mutex);
+		if (width > 0) {
+			int ret;
+			struct current_cursor_pos yx;
 
-		if (ret == ERR)
-			readline_error(0, "handle_key: wmove");
+			yx = term_get_pos(ctx->act);
+
+			mutex_lock(&g_puts_mutex);
+			ret = wmove(ctx->act, yx.cury, yx.curx + width);
+			mutex_unlock(&g_puts_mutex);
+
+			if (ret == ERR)
+				readline_error(0, "handle_key: wmove");
+		}
 	} else {
 		ctx->buffer[ctx->bufpos] = wc;
 		ctx->bufpos++;
 		ctx->numins++;
+		ctx->vispos += width;
 		readline_waddch(ctx->act, wc);
 	}
 
@@ -526,11 +586,12 @@ new_session(CSTRING prompt)
 	ctx->buffer      = xcalloc(g_readline_bufsize + 1, sizeof(wchar_t));
 	ctx->bufpos      = 0;
 	ctx->insert_mode = false;
-	ctx->numins      = 0;
 	ctx->no_bufspc   = false;
+	ctx->numins      = 0;
 	ctx->prompt      = sw_strdup(prompt);
 	ctx->prompt_size = (int) strlen(squeeze_text_deco(prompt_copy));
 	ctx->tc          = readline_tab_comp_ctx_new();
+	ctx->vispos      = 0;
 
 	free(prompt_copy);
 
@@ -783,7 +844,7 @@ process(volatile struct readline_session_context *ctx)
 			session_destroy(ctx);
 			return NULL;
 		default:
-			if (iswprint(wc))
+			if (iswprint(wc) && !is_combined(wc))
 				handle_key(ctx, wc, true);
 			break;
 		}
@@ -879,13 +940,22 @@ readline(CSTRING prompt)
 
 	switch (setjmp(g_readline_loc_info)) {
 	case READLINE_PROCESS:
-		g_readline_loop = true;
-		g_resize_requested = false;
-		g_suggs_mode = false;
-		g_hist_next = false;
-		g_hist_prev = false;
+		g_hist_next		= false;
+		g_hist_prev		= false;
+		g_readline_loop		= true;
+		g_resize_requested	= false;
+		g_suggs_mode		= false;
 		ctx = new_session(prompt);
 		return process(ctx);
+	case READLINE_RESTART:
+		debug("%s: restarting...", __func__);
+		break;
+	case READLINE_TERMINATE:
+		debug("%s: terminating...", __func__);
+		break;
+	default:
+		sw_assert_not_reached();
+		break;
 	}
 
 	session_destroy(ctx);
@@ -923,7 +993,7 @@ readline_mouse_init(void)
 		    "WHEEL"))
 			report_wheel_events();
 		else
-			err_log(EINVAL, "readline_mouse_init: 'mouse_events'");
+			err_log(EINVAL, "%s: 'mouse_events'", __func__);
 	} else {
 		(void) mousemask(0, NULL);
 	}
