@@ -18,8 +18,8 @@ void	theme_deinit(void);
 bool		 theme_bool(const char *, bool);
 char		*Theme_mod(const char *);
 const char	*Theme(const char *);
-int		 theme_item_install(const char *name, const char *value);
-int		 theme_item_undef(const char *name);
+errno_t		 theme_item_install(const char *name, const char *value);
+errno_t		 theme_item_undef(const char *name);
 long int	 theme_integer(const struct integer_context *);
 short int	 theme_color(const char *, short int);
 void		 theme_create(const char *path, const char *mode);
